% This file estimates the IRF of proxy SVAR for a monetary SVAR
% This program is adapted from replication files of Mertens and Ravn, 2014, 'A
% Reconciliation of SVAR and Narrative Estimates of Tax Mulitpliers', JME

% Note: the number of bootstrap replications is set to 100 for speed (the paper uses 10000)
% In my case, ffr is tax, lip is G, lcpi is Y, lcons is W.
clear all; close all;
clc;


% Load the data
DATA = xlsread('data_q.xlsx');


%  Proxy SVAR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
 DATA= DATA(57:end-4,:); % 57:1983.0 
 

 dates = DATA(:,1);
 rr = DATA(:,2); % 2:rr,19:cum_rr
 ffr = DATA(:,3);
 hour = DATA(:,9); % 6, 9, 12
 emp =  DATA(:,7); % 7, 10
 gdp = DATA(:,17);
 cpi = log(DATA(:,18));

 VARNA.vars  = [ffr 100*log(gdp./emp) 100*log(gdp./hour) cpi];
 VARNA.p = 4;
 [T,n]  = size(VARNA.vars);
 VARNA.DET = ones(T,1);
 VARNA.irhor = 21;

 VARNA.mshocks = rr;
 
 % Demean the narrative shocks
  for j=1:size(VARNA.mshocks,2)
     VARNA.mshocks(VARNA.mshocks(:,j)~=0,j)=(VARNA.mshocks(VARNA.mshocks(:,j)~=0,j)-mean(VARNA.mshocks(VARNA.mshocks(:,j)~=0,j)));
  end
  
DAT.TRY = 0.1746; % Average ratio of federal tax revenues to GDP
DAT.GY  = 0.0981; % Average ratio of federal expenditures to GDP

VARNA.mshocksize =  1;
VARNA.gshocksize =  0.01/DAT.GY; % left in because one of the other programs needs it
VARNA = doPVAR(VARNA);
nboot = 1000; % Note:  the paper uses 10000
clevel = 68; % Confidence Level
VARNAbs = doPVARbs(VARNA,DAT,nboot,clevel);


% Plot Impulse Response to a Monetary Shock
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   H = 20;
    weight=zeros(H+1,H+1);
    weight(1,1)=1; % End-points are not MAs
    weight(H+1,H+1)=1;
    
    for i=2:H
        weight(i,i-1:i+1)=[1/3 1/3 1/3]; % Centred three-period moving average
    end
    
    VARNA.irs = VARNAbs.irsM; % Point estimate vs. median


         f=figure;
           box off
        
         plot(-VARNA.irs(:,2),'LineWidth',2,'Color', [0 0 0.5]);
        hold on       
        plot(-VARNAbs.irsH(:,2),'LineWidth',1,'Color',[0 0 0.5],'LineStyle','--' ); 
        hold on
        plot(-VARNAbs.irsL(:,2),'LineWidth',1,'Color',[0 0 0.5],'LineStyle','--' ); 
        hold on
        %axis([0 48 FIG.axes(4,1) FIG.axes(4,2)])
        hline(0,'k-')
        ti=title('GDP/Employment');
        xl=xlabel('quarters');
        yl=ylabel('percent');
        xlim([1 20])
    
         set([xl,yl],  'FontSize',16);
        set([ti], 'FontName', 'Times New Roman','FontSize',16); 
        grid on
        
        set(gcf,'PaperPositionMode','auto')
        print -depsc2 prod2.eps    
        
        
           f=figure;
           box off
        
         plot(-VARNA.irs(:,3),'LineWidth',2,'Color', [0 0 0.5]);
        hold on       
        plot(-VARNAbs.irsH(:,3),'LineWidth',1,'Color',[0 0 0.5],'LineStyle','--' ); 
        hold on
        plot(-VARNAbs.irsL(:,3),'LineWidth',1,'Color',[0 0 0.5],'LineStyle','--' ); 
        hold on
        %axis([0 48 FIG.axes(4,1) FIG.axes(4,2)])
        hline(0,'k-')
        ti=title('GDP/Hours');
        xl=xlabel('quarters');
        yl=ylabel('percent');
        xlim([1 20])
    
        set([xl,yl],  'FontSize',16);
        set([ti], 'FontName', 'Times New Roman','FontSize',16); 
        grid on
        
        set(gcf,'PaperPositionMode','auto')
        print -depsc2 prod1.eps    
        
        
        

         
         
  
        